/*
 * cocot - COde COnverter on Tty
 *
 * Copyright (c) 2002  IWAMURO Motonori
 * All rights reserved.
 */

#include <stdio.h>
#include <unistd.h>
#include <utmp.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <getopt.h>

#include "config.h"
#include "init.h"
#include "loop.h"

static volatile void
usage(int argc, char *argv[])
{
    fprintf(stderr,
	    "Usage: %s [OPTIONS] COMMAND ARG1 ARG2 ...\n"
	    "\n"
	    "Options:\n"
	    "    -o LOGFILE    loging all output of command.\n"
	    "    -a            append log file.\n"
	    "    -t TERM_CODE  character code of terminal.\n"
	    "    -p PROC_CODE  character code of command process.\n",
	    argv[0]);
    exit(1);
}

int
main(int argc, char *argv[])
{
    int c;
    char *logfn = NULL;
    char *logmd = "w";
    FILE *logfp = NULL;
    char *term_code = DEFAULT_TERM_CODE;
    char *proc_code = DEFAULT_PROC_CODE;

    int mfd, sfd;
    int status;
    pid_t pid;

    if (argc == 1)
	usage(argc, argv);
    while ((c = getopt(argc, argv, "ao:t:p:")) != -1) {
	switch (c) {
	case 'a':
	    logmd = "a";
	    break;
	case 'o':
	    logfn = optarg;
	    break;
	case 't':
	    term_code = optarg;
	    break;
	case 'p':
	    proc_code = optarg;
	    break;
	}
    }
    if (logfn) {
	if ((logfp = fopen(logfn, logmd)) == NULL)
	    fatal("Can't open file '%s' (%s).", logfn, strerror(errno));
	setvbuf(logfp, NULL, _IONBF, 0);
    }
    init(&mfd, &sfd);
    if ((pid = fork()) < 0) {
	/* error */
	fatal("Can't fork process");
    } else if (pid == 0) {
	/* child */
	close(mfd);
	if (logfp)
	    fclose(logfp);
	login_tty(sfd);
	execvp(argv[optind], argv + optind);
	fatal("Can't exec process");
    }
    /* parent */
    close(sfd);
    loop(mfd, logfp, term_code, proc_code);
    close(mfd);
    if (logfp)
	fclose(logfp);
    wait(&status);
    done();
    return 0;
}
